package com.muyu.server.service.impl;

import cn.hutool.core.map.MapUtil;
import com.alibaba.druid.pool.DruidDataSource;
import com.muyu.cache.EnterpriseCacheService;
import com.muyu.cloud.common.many.datasource.ManyDataSource;
import com.muyu.cloud.common.many.datasource.constents.DatasourceContent;
import com.muyu.cloud.common.many.datasource.domain.model.DataSourceInfo;
import com.muyu.cloud.common.many.datasource.factory.DruidDataSourceFactory;
import com.muyu.cloud.common.many.datasource.init.InitDataSource;
import com.muyu.cloud.common.many.datasource.role.DynamicDataSource;
import com.muyu.cloud.common.saas.domain.Datasource;
import com.muyu.cloud.common.saas.domain.model.EntInfo;
import com.muyu.cloud.common.saas.exception.SaaSException;
import com.muyu.common.core.constant.SecurityConstants;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.SpringUtils;
import com.muyu.common.domain.Enterprise;
import com.muyu.common.system.domain.Business;
import com.muyu.common.system.domain.SysUser;
import com.muyu.common.system.remote.RemoteUserService;
import com.muyu.common.util.PageUtils;
import com.muyu.server.mapper.EnterpriseMapper;
import com.muyu.server.service.EnterpriseService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.support.EncodedResource;
import org.springframework.jdbc.datasource.init.ScriptUtils;
import org.springframework.stereotype.Service;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;

/**
 * 企业运营 服务层处理
 * @author yuping
 * @package com.muyu.server.service.impl
 * @name EnterpriseServiceImpl
 * @Data 2024-09-26 20:23:21
 */
@Service
@Log4j2
public class EnterpriseServiceImpl implements EnterpriseService {

    @Autowired
    private EnterpriseMapper enterpriseDao;

    @Autowired
    private RemoteUserService remoteUserService;

    @Autowired
    private ManyDataSource manyDataSource;


    /**
     * 分页查询企业运营信息
     * @param param
     * @return
     */
    @Override
    public PageUtils selectEnterprise(Map param) {
        List<HashMap> list = new ArrayList<>();
        long count = enterpriseDao.selectEnterpriseCount();
        if (count > 0){
            list = enterpriseDao.selectEnterprise(param);
        }
        int page = MapUtil.getInt(param, "page");
        int length = MapUtil.getInt(param,"length");
        PageUtils pageUtils = new PageUtils(list, count, page, length);
        return pageUtils;
    }


    /**
     * 新增企业信息
     * @param enterprise
     * @return
     */
    @Override
    public int insert(Enterprise enterprise) {

        int rows = enterpriseDao.insert(enterprise);
        if (rows == 0){
           log.info("新增失败");
        }

        Business business = new Business();
        business.setFirmName(enterprise.getEnterpriseName());
        business.setDatabaseName(enterprise.getEnterpriseDatabaseName());
        remoteUserService.settlementEnterpriseInfo(business, SecurityConstants.INNER);

        SysUser sysUser = new SysUser();
        sysUser.setDatabaseName(enterprise.getEnterpriseDatabaseName());
        sysUser.setUserName(enterprise.getEnterpriseName());
        sysUser.setCreateTime(new Date());
        remoteUserService.addUser(sysUser, SecurityConstants.INNER);
        String createDatabaseUrl="jdbc:mysql://"+ DatasourceContent.IP+":"+DatasourceContent.PORT+"?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8";
        String createDatabaseSql = "CREATE DATABASE IF NOT EXISTS " + enterprise.getEnterpriseDatabaseName() + ";";

        try (Connection adminConn = DriverManager.getConnection(createDatabaseUrl, DatasourceContent.USER_NAME, DatasourceContent.PASSWORD);
             Statement stmt = adminConn.createStatement()) {

            boolean success = stmt.execute(createDatabaseSql);
            if (success) {
                log.info("数据库 {} 创建失败", enterprise.getEnterpriseDatabaseName());

            } else {
                log.warn("数据库 {} 创建成功", enterprise.getEnterpriseDatabaseName());

                // 切换到新的数据库连接
                Connection connection = null;
                String url = "jdbc:mysql://47.101.53.251:3306/" + enterprise.getEnterpriseDatabaseName() + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8";
                String user = "root";
                String pwd = "Lw030106";
                String driverClassName = "com.mysql.cj.jdbc.Driver";

                try {
                    Class.forName(driverClassName);
                    try (Connection connection1 = DriverManager.getConnection(url, user, pwd)) {
                        ClassPathResource rc = new ClassPathResource("saas.sql");
                        EncodedResource er = new EncodedResource(rc, "utf-8");
                        ScriptUtils.executeSqlScript(connection1, er);

                    } catch (SQLException e) {
                        throw new RuntimeException("无法执行SQL脚本", e);
                    }
                } catch (ClassNotFoundException e) {
                    throw new RuntimeException("链接不到数据源驱动", e);
                }

            }

        } catch (SQLException e) {
            e.printStackTrace();
            log.error("连接数据库时发生错误或创建数据库失败", e);
        }



        DruidDataSourceFactory druidDataSourceFactory = SpringUtils.getBean(DruidDataSourceFactory.class);
        DynamicDataSource dynamicDataSource = SpringUtils.getBean(DynamicDataSource.class);
        for (EntInfo entInfo : manyDataSource.dataSourceInfoList()) {
            DataSourceInfo dataSourceInfo = DataSourceInfo.hostAndPortBuild(
                    entInfo.getEntCode(), entInfo.getIp(), entInfo.getPort()
            );
            DruidDataSource druidDataSource = druidDataSourceFactory.create(dataSourceInfo);
            dynamicDataSource.put(dataSourceInfo.getKey(), druidDataSource);
            log.info("存储数据连接池为：key:{}",dataSourceInfo.getKey());
        }
        log.info("企业信息插入成功");
        return rows;
    }


    /**
     * 根据编号查询企业信息
     * @param enterpriseId
     * @return
     */
    @Override
    public HashMap searchById(int enterpriseId) {
        HashMap map = enterpriseDao.searchById(enterpriseId);
        return map;
    }


    /**
     * 修改企业信息
     * @param enterprise
     * @return
     */
    @Override
    public int updateEnterprise(Enterprise enterprise) {
        int rows = enterpriseDao.updateEnterprise(enterprise);
        return rows;
    }


    /**
     * 删除企业信息
     * @param ids
     * @return
     */
    @Override
    public int deleteByIds(Integer[] ids) {
        if (ids == null || ids.length == 0) {
            throw new IllegalArgumentException("请选择要删除的企业名称编号");
        }
        Integer[] idList = Arrays.asList(ids).toArray(new Integer[0]);
        int rows = enterpriseDao.deleteByIds(idList);
        return rows;

    }
}
